# -*- coding: utf-8 -*-
"""
web.py
======

Context decorator for producing figures suitable for web pages.

PNG format for image file are used, because in web pages the pixel unit
and speed are most important.

NOTE:
DPI or PPI is not important for PNG images. but it is critical in
matplotlib. Matplotlib uses physical unit for its plot.
Therefore, for same number of pixels, higher DPI will require smaller
physical size in inch. But small physical size will cause plotting
problems.
It is obvious now we should choose a DPI that best matches the current working
screen, so that matplotlib will draw things nicely.

"""

from .general import MPLdecorator
from .layout import GOLDEN_RATIO, point2inch
from .colors import brewer_set1

__all__ = ['web_decorator',]

# Constants for web
_width_normal_px = 440
save_dpi = 150
width_normal = point2inch(_width_normal_px, save_dpi)
width_tiny = 0.5 * width_normal
width_small = 0.8 * width_normal
width_large = 1.2 * width_normal
width_large2 = 1.5 * width_normal
width_huge = 1.8 * width_normal

# Default ratio for a single plot figure
# I prefer a little higher than goden ratio, from 0.618 to 0.68
height_width_ratio = GOLDEN_RATIO * 1.1  # = height / width

_width = width_normal
_height = _width * height_width_ratio

_line_width = 0.8

_params = {'font.family' : 'sans-serif',
           'font.serif' : ['Bitstream Vera Serif', 'Computer Modern Roman'],
           'font.sans-serif' : ['Helvetica', 'Arial', 'Lucida Grande'],
           'font.size' : 7,
           'font.weight' : 'normal',
           'text.usetex' : True,
           # To force LaTeX use Helvetica fonts.
           'text.latex.preamble': [
                                    r'\usepackage{siunitx}',
                                    r'\sisetup{detect-all}',
                                    r'\usepackage{helvet}',
                                    r'\usepackage[eulergreek,EULERGREEK]{sansmath}',
                                    r'\sansmath'],

           'axes.color_cycle': brewer_set1,
           'axes.labelsize' : 'medium',
           'axes.labelweight' : 'normal',
           'axes.linewidth' : _line_width,

           'figure.figsize' : (_width, _height),
           'figure.subplot.left' : 0.125,
           'figure.subplot.right' : 0.95,
           'figure.subplot.bottom' : 0.1,
           'figure.subplot.top' : 0.95,

           'savefig.dpi' : save_dpi,
           'savefig.format': 'png',
           #'savefig.bbox': 'tight',
           # this will crop white spaces around images that will make
           # width/height no longer the same as the specified one.

           'legend.fontsize' : 'small',
           'legend.frameon' : False,
           'legend.numpoints' : 1,
           'legend.handlelength' : 2,
           'legend.scatterpoints' : 1,
           'legend.labelspacing' : 0.5,
           'legend.markerscale' : 0.9,
           'legend.handletextpad' : 0.5,  # pad between handle and text
           'legend.borderaxespad' : 0.5,  # pad between legend and axes
           # pad between legend and legend content
           'legend.borderpad' : 0.5,
           # pad between each legend column
           'legend.columnspacing' : 1,

           #'text.fontsize' : 'medium',

           'xtick.major.size' : 3,
           #'xtick.minor.size' : 2,
           'xtick.major.width' : _line_width,
           #'xtick.minor.width' : 0.5,
           'xtick.major.pad' : 2,
           #'xtick.minor.pad' : 4,
           #'xtick.color' : k,
           'xtick.labelsize' : 'medium',
           #'xtick.direction' : 'in',

           'ytick.major.size' : 3,
           #'ytick.minor.size' : 2,
           'ytick.major.width' : _line_width,
           #'ytick.minor.width' : 0.5,
           'ytick.major.pad' : 2,
           #'ytick.minor.pad' : 4,
           #'ytick.color' : k,
           'ytick.labelsize' : 'medium',
           #'ytick.direction' : 'in',

           'lines.linewidth' : _line_width,
           'lines.markersize' : 3,
           #'lines.markeredgewidth' : 0,
           # 0 will make line-type markers, such as '+', 'x', invisible
          }

web_decorator = MPLdecorator(_params)




# -*- coding: utf-8 -*-
"""
styles.py
=========

Toolkits and constants for plotting styles.

"""

import itertools

from .colors import almost_black, brewer_set1

__all__ = ['lines', 'markers', 'nextlinestyle', 'linestyle_generator']


_colors = itertools.cycle(brewer_set1)

_line_styles = ['-', '--', '-.', ':']
lines = itertools.cycle(_line_styles)

_markers = ['o', 's', 'v', '^', 'D', 'p', 'h', '<', '>']
markers = itertools.cycle(_markers)

# for making hollow markers, use it together with marker_types
_markersh = ['o', 'o', 's', 's', 'v', 'v', '^', '^', 'D', 'D']
markersh = itertools.cycle(_markersh)

_marker_types = [False, True]  # True for hollow markers
marker_types = itertools.cycle(_marker_types)


def linestyle_generator(colors=brewer_set1, lines=_line_styles,
                        markers=_markers, hollow_styles=_marker_types):
    """
    Generate a dict for configuring plot line styles.

    The default line style is markers linked by lines, both styles are cycled,
    and hollow markers are included.

    Usage::

      linestyles = linestyle_generator()
      linestyle = linestyles.next()

    Note that all inputs should be iterable objects.
    To exclude one style, just pass empty list or ``None`` to the corresponding argument.
    To include one style but not cycle, just pass a single-element list.
    For example, pass ``lines=['-'] to apply solid line style to all line arts.
    Similarly, passing ``markers=['o']`` will apply filled circle only.

    :param colors: list of colors, (tuple of RGB or #FFFFFF
                or color name string)
    :type colors: list of tuple or list of string or ``None``
    :param lines: list of line styles within ('-', '--', '-.', ':')
    :type lines: list of string,  or ``None``
    :param markers: list of markers (see matplotlib for available markers)
    :type markers: list of charaters,  or ``None``
    :param hollow_styles: True for hollow markers, False for filled markers.
    :param hollow_styles: list of bool,  or ``None``
    :return: dict of parameters of linestyle
    :rtype: dict
    """

    # If both lines and markers are empty or None, do nothing
    is_nothing = False
    if not lines and not markers:
        is_nothing = True

    if colors:
        color_cycle = itertools.cycle(colors)
    else:  # default line color is almost_black
        color_cycle = itertools.cycle([almost_black])

    if lines:
        line_cycle = itertools.cycle(lines)
    else:  # empty list or None supplied, disable line connection
        line_cycle = itertools.cycle([''])

    if markers and hollow_styles:  # solid and hollow markers
        full_markers = itertools.product(markers, hollow_styles)
    elif markers and not hollow_styles:  # all solid markers
        full_markers = itertools.product(markers, [None])
    else:  # no markers
        full_markers = itertools.product(['None'], [None])
    marker_cycle = itertools.cycle(full_markers)

    while True:
        if not is_nothing:
            color = color_cycle.next()
            linestyle = line_cycle.next()
            marker, hollow = marker_cycle.next()
            if hollow is None:  # only filled markers
                mew = 1
                mec = color
                mfc = color
            elif hollow:  # make hollow markers
                mew = 1
                mec = color
                mfc = 'None'
            else:  # otherwise, make filled markers
                mew = 1
                mec = color
                mfc = color
            yield {'color': color, 'linestyle':linestyle,
                   'marker':marker, 'mew':mew, 'mec':mec, 'mfc':mfc}
        else:
            yield {}


def nextlinestyle(no_line=False, is_line=True, is_marker=True, is_hollow=True):
    """
    Generate a dict for configuring plot line style.

    NOTE: this function has a serious problem.
    As long as mpltex is imported,
    the beginning of the sequence cannot be configured later.
    This function is obsolete since version 0.2, please use linestyle_generator instead.

    The default line style is markers linked by lines, both styles are cycled,
    and hollow markers are included.
    Disable cycling one style by set the corresponding flag to False.

    :param no_line: if True, only markers, no line segments
    :type no_line: bool
    :param is_line: if True, cycle line styles. No effect when no_line=True.
    :type is_line: bool
    :param is_marker: if True, cycle markers.
    :type is_marker: bool
    :param is_hollow: if True, cycle markers including hollow styles.
    :param is_hollow: bool
    :return: dict of parameters of linestyle
    :rtype: dict
    """
    color = _colors.next()
    linestyle = lines.next()
    if(is_hollow):
        marker = markersh.next()
    else:
        marker = markers.next()
    marker_type = marker_types.next()
    if marker_type:  # hollow mark
        mew = 1
        mec = color
        mfc = 'None'
    else:
        mew = 1
        mec = color
        mfc = color

    linestyles = {}
    if (is_line):
        linestyles['linestyle'] = linestyle
    if (no_line):  # overwrite is_line if no_line is True
        linestyles['linestyle'] = ''
    if (is_marker):
        linestyles['marker'] = marker
        linestyles['mew'] = 0  # make the marker no edge
    if (is_hollow):
        linestyles['color'] = color
        linestyles['marker'] = marker
        linestyles['mew'] = mew
        linestyles['mec'] = mec
        linestyles['mfc'] = mfc
    return linestyles



# -*- coding: utf-8 -*-
"""
layout.py
=========

Toolkits and constants for plotting layout.

"""

__all__ = ['point2inch', 'inch2point', 'GOLDEN_RATIO', 'GOLDEN_RATIO2',]


GOLDEN_RATIO = 0.618
GOLDEN_RATIO2 = 1.618

def point2inch(npt, dpi=72.0):
     """
     Point to inch converter.

     :param npt: number of points
     :type npt: integer
     :return: number of inches corresponding to :param:`npt`
     :type: double
     """
     return 1.0 * npt / dpi


def inch2point(inch, dpi=72.0):
     """
     Inch to point converter.

     :param inch: the length in unit inch
     :type inch: double
     :return: number of points corresponding to :param:`inch`
     :rtype: integer
     """
     return int(dpi * inch)
